/***************************************************************************
 *   Copyright (C) 2007 by Antony Martin                                   *
 *   martin.antony@yahoo.fr                                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

/* Cree le : 21 decembre 2006
   derniere modification le 10/02/2007 */

#ifndef __SCECAM_H__
#define __SCECAM_H__


/* constantes propres a Cam */
#define SCE_CAM_MODE_FREE 1
#define SCE_CAM_MODE_FIX 2


typedef struct sce_sviewport SCE_SViewport;
struct sce_sviewport
{
    int x, y;
    int w, h;
}; 


/* structure d'une camera */
typedef struct sce_scamera SCE_SCamera;
struct sce_scamera
{
    SCE_TVector3 p; /* position */
    SCE_TVector3 v; /* vecteur de vue */
    SCE_TVector3 i; /* inclinaison */
    
    /* viewport de la camera */
    SCE_SViewport viewport;
    
    /* mode de la camera */
    unsigned int mode;
    
    
    
    /* pour le mode fixation */
    /* distance par rapport au point fixe */
    float fix_lenght;
    
    
    /* booleen, true: utilisation de l'inertie */
    int use_rot_inert;
    int use_mov_inert;
    
    /* booleen, true: mouvements/rotations additionnels */
    int use_additionnal_mov;
    int use_additionnal_rot;
    
    
    /* angles de rotation additionnels */
    float z, x;
    
    /* angles de rotation reels */
    float relz, relx;
    
    /* angles qui ont subit l'inertie */
    float iz, ix;
    
    
    /* deplacements: avant-arriere;
                     gauche-droite.
       (ont subit l'inertie, valeurs a appliquer lors du calcul du mouvement) */
    float left, front;
    /* deplacements reels */
    float l_rel, f_rel;
    /* inerties (manques) des mouvements */
    float l_miss, f_miss;
    /* nombre de passages deja effectues (voir methode de l'inertie) */
    int f_fois, l_fois;
    
    
    /* inerties */
    float r_in;     /* inertie de rotation */
    int r_pass;     /* nombre de passes avant (appels de fonction)
                       l'arret de l'application de l'inertie (rotation) */
    
    float m_in;     /* inertie de mouvement */
    int m_pass;     /* nombre de passes (appels de fonction) avant
                       l'arret de l'application de l'inertie (mouvement) */
    
    /* nombre de passes courantes */
    int m_fois;
    int r_fois;
};



/* initialise une camera a ses valeurs par defaut */
void SCE_Cam_Init(SCE_SCamera*);



/* defini la zone de l'ecran ou la camera sera representee */
void SCE_Cam_SetViewport(SCE_SCamera*, int, int, int, int);


/* defini les valeurs de l'inertie du mouvement de la camera */
void SCE_Cam_SetMoveInert(SCE_SCamera*, float, int);

/* defini les valeurs de l'inertie de la rotation de la camera */
void SCE_Cam_SetRotateInert(SCE_SCamera*, float, int);

/* defini le mode de la camera */
void SCE_Cam_SetMode(SCE_SCamera*, unsigned int);

/* utilisation de l'inertie de rotation: oui, non */
void SCE_Cam_UseRotInert(SCE_SCamera*, int);

/* utilisation de l'inertie de mouvement: oui, non */
void SCE_Cam_UseMovInert(SCE_SCamera*, int);



/* defini le point de fixation de la camera
   (calculs adaptes en fonction du mode de la camera) */
void SCE_Cam_SetPointView(SCE_SCamera*, SCE_TVector3);


/* defini les prochaines rotations qu'effectuiera la camera specifiee
   (rotation adaptives en fonction du mode de la camera) */
void SCE_Cam_Rotate(SCE_SCamera*, float, float);


/* defini le prochain traget de la camera (avant/arriere ; latteral) */
void SCE_Cam_Move(SCE_SCamera*, float, float);


/* calcul les nouvelles coordonnees de la camera specifiee */
void SCE_Cam_Compute(SCE_SCamera*);

/* defini la camera comme camera active */
void SCE_Cam_SetActive(const SCE_SCamera* const);


#endif /* guard */
