/***************************************************************************
 *   Copyright (C) 2007 by Antony Martin                                   *
 *   martin.antony@yahoo.fr                                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

/* Cree le : 15 decembre 2006
   derniere modification le 08/03/2007 */


#include <SCE/SCEMinimal.h>

#include <SCE/core/SCECore.h>


/* initialise opengl avec les options de configuration minimums */
static int SCE_CInitGL(void)
{
    GLenum error;
    
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    
    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();
    
    /* ne joue que sur l'unite de texture active, donc la 1 */
    glMatrixMode(GL_TEXTURE);
    glLoadIdentity();
    
    glMatrixMode(GL_MODELVIEW);
    
    glClearColor(0.0, 0.0, 0.0, 0.0);
    /*glClearIndex(0.0);*/ /* ... */
    glClearDepth(1.0);
    glClearStencil(0);
    glClearAccum(0.0, 0.0, 0.0, 0.0);
    
    glShadeModel(GL_SMOOTH);
    
    error = glGetError();
    
    if(error == GL_NO_ERROR)
    {
        return (SCE_OK);
    }
    else
    {
        Logger_Log(error);
        Logger_LogMsg((char*)gluErrorString(error));
    }
    
    return (SCE_ERROR);
}


int SCE_CInit(SCEflags flags)
{
    int use_cg = 0;
    GLenum error;
    
    
    /* verification des flags */
    if(flags & SCE_USE_CG_SHADERS)
        use_cg = 1;
    
    if(SCE_CInitGL() < 0)
        return (SCE_ERROR);
    
    error = glewInit();
    if(error != GLEW_OK)
    {
        Logger_Log((signed)error);
        Logger_LogMsg("cannot initialize GLEW : %s",
                      (char*)glewGetErrorString(error));
        return (SCE_ERROR);
    }
    
    /* initialisation des modules du coeur */
    /* (et y'en a qu'un...) */
    if(SCE_CSupportInit() < 0)
    {
        Logger_LogSrc();
        return (SCE_ERROR);
    }
    
    /* initialisation du gestionnaire de shaders */
    if(SCE_CInitShaders(use_cg) < 0)
    {
        Logger_LogSrc();
        return (SCE_ERROR);
    }
    
    return (SCE_OK);
}


int SCE_CQuit(void)
{
    /* pour l'instant, c'est tout ce qu'il y a a faire pour quitter le coeur */
    SCE_CQuitShaders();
    
    return (SCE_OK);
}




void SCE_CClear(const GLbitfield mask)
{
    glClear(mask);
}


void SCE_CSetState(SCEenum state, int active)
{
    if(active)
    {
        glEnable(state);
    }
    else
    {
        glDisable(state);
    }
}

void SCE_CSetState2(SCEenum state, SCEenum state2, int active)
{
    if(active)
    {
        glEnable(state);
        glEnable(state2);
    }
    else
    {
        glDisable(state);
        glDisable(state2);
    }
}

void SCE_CSetState3(SCEenum state, SCEenum state2, SCEenum state3, int active)
{
    if(active)
    {
        glEnable(state);
        glEnable(state2);
        glEnable(state3);
    }
    else
    {
        glDisable(state);
        glDisable(state2);
        glDisable(state3);
    }
}

void SCE_CSetState4(SCEenum state, SCEenum state2,
                    SCEenum state3, SCEenum state4, int active)
{
    if(active)
    {
        glEnable(state);
        glEnable(state2);
        glEnable(state3);
        glEnable(state4);
    }
    else
    {
        glDisable(state);
        glDisable(state2);
        glDisable(state3);
        glDisable(state4);
    }
}

void SCE_CSetState5(SCEenum state, SCEenum state2, SCEenum state3,
                    SCEenum state4, SCEenum state5, int active)
{
    if(active)
    {
        glEnable(state);
        glEnable(state2);
        glEnable(state3);
        glEnable(state4);
        glEnable(state5);
    }
    else
    {
        glDisable(state);
        glDisable(state2);
        glDisable(state3);
        glDisable(state4);
        glDisable(state5);
    }
}

void SCE_CSetState6(SCEenum state, SCEenum state2, SCEenum state3, SCEenum state4,
                    SCEenum state5, SCEenum state6, int active)
{
    if(active)
    {
        glEnable(state);
        glEnable(state2);
        glEnable(state3);
        glEnable(state4);
        glEnable(state5);
        glEnable(state6);
    }
    else
    {
        glDisable(state);
        glDisable(state2);
        glDisable(state3);
        glDisable(state4);
        glDisable(state5);
        glDisable(state6);
    }
}

