/***************************************************************************
 *   Copyright (C) 2007 by Antony Martin                                   *
 *   martin.antony@yahoo.fr                                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

/* Cree le : 19 janvier 2007
   derniere modification le 04/03/2007 */


#include <SCE/SCEMinimal.h>

#include <SCE/utils/SCEString.h>

#include <SCE/core/SCECore.h>

#include <SCE/interface/SCEScene.h>


#define SCE_GETSTATE 0
#define SCE_SETENABLE 1
#define SCE_SETDISABLE 2


/* macro par defaut a utiliser en premier dans chaque fonction
   pour verifier si le gestionnaire de scenes est initialise */
#define SCE_Scene_CheckState()\
    if(!SCE_Scene_IsInit())\
    {\
        Logger_Log(SCE_INVALID_OPERATION);\
        Logger_LogMsg("you can't use the scene manager "\
                      "if this is not initialized");\
        \
        return (NULL);\
    }\


/* fonction de positionnement de l'etat actif du gestionnaire de scenes
   renvoie l'etat du gestionnaire de scenes */
static int SCE_Scene_Initialized(int new_state)
{
    static int state=0;
    
    switch(new_state)
    {
        case SCE_GETSTATE:
            break;
        
        case SCE_SETENABLE:
            state = 1;
            break;
        
        case SCE_SETDISABLE:
            state = 0;
            break;
        
        #ifdef __SCE_DEBUG__
        default:
            Logger_Log(SCE_INVALID_ARG);
            Logger_LogMsg("unknow new status!");
            break;
        #endif
    }
    
    return (state);
}


int SCE_Scene_IsInit(void)
{
    return (SCE_Scene_Initialized(SCE_GETSTATE));
}


int SCE_Scene_Init(void)
{
    SCE_Scene_Initialized(SCE_SETENABLE);
    return (SCE_OK);
}

int SCE_Scene_Quit(void)
{
    SCE_Scene_Initialized(SCE_SETDISABLE);
    return (SCE_OK);
}



SCE_SScene* SCE_Scene_Create(void)
{
    SCE_SScene *scene = NULL;
    
    
    SCE_Scene_CheckState();
    
    scene = SCE_malloc(sizeof *scene);
    if(scene == NULL)
    {
        Logger_LogSrc();
        return (NULL);
    }
    
    return (scene);
}


int SCE_Scene_Delete(SCE_SScene *scene)
{
    if(SCE_free(scene) < 0)
        return (-1);
    else
        return (SCE_OK);
}





